# -*- coding: utf-8 -*-
# @Time    : 2023/3/11 10:45
# @Author  : Euclid-Jie
# @File    : EuclidDataTools.py
import os
import pandas as pd


class EuclidCsvTools:
    """
    this class include tools used to precess csv file
    """

    def __init__(self, subFolder: str = None, FileName: str = 'DemoOut.csv'):
        # para init
        self.subFolder = subFolder
        self.FileName = FileName
        self.FullFilePath = None
        self.FullFolderPath = None

    def path_clear(self):
        """
        get the full folder path and full file path
        :return:
        """
        if self.subFolder:
            if '\\' in self.subFolder:
                self.FullFolderPath = os.getcwd() + self.subFolder
            else:
                self.FullFolderPath = os.getcwd() + '\\' + self.subFolder
        else:
            self.FullFolderPath = os.getcwd()
        self.FullFilePath = os.path.join(self.FullFolderPath, self.FileName)
        print('文件将存储在: {}'.format(self.FullFilePath))

    def saveCsvFile(self, df, append=False):
        """
        save data to csv
        :param df: pd.DataFrame
        :param append: True(append save) or False(overwrite)
        :return:
        """
        if not self.FullFilePath:
            self.path_clear()

        if os.path.exists(self.FullFolderPath):
            # folder path exit
            if append:
                self.writeDf2Csv(df, self.FullFilePath)
            else:
                df.to_csv(self.FullFilePath, encoding='utf_8_sig', index=False)
        else:
            # no dir exists then make one and save data to csv
            os.mkdir(self.FullFolderPath)
            if append:
                self.writeDf2Csv(df, self.FullFilePath)
            else:
                df.to_csv(self.FullFilePath, encoding='utf_8_sig', index=False)

    @classmethod
    def writeDf2Csv(cls, df, FullFilePath):
        if not os.path.exists(FullFilePath):
            # write out a new file with header
            df.to_csv(FullFilePath, mode='w', encoding='utf_8_sig', header=True, index=False)
        else:
            # write after a exist file without header
            df.to_csv(FullFilePath, mode='a', encoding='utf_8_sig', header=False, index=False)


class CsvClient(EuclidCsvTools):
    def __init__(self, subFolder: str = None, FileName: str = 'DemoOut.csv'):
        """
        :param subFolder:
        :param FileName:
        """
        super().__init__(subFolder=subFolder, FileName=FileName)
        if FileName[-4:] != '.csv':
            self.FileName = self.FileName + '.csv'
        self.path_clear()

    def insert_one(self, data):
        if isinstance(data, dict):
            data = pd.DataFrame([data])
        elif isinstance(data, pd.DataFrame):
            pass
        else:
            raise TypeError("传入参数仅支出dict和pd.DataFrame")
        self.saveCsvFile(df=data, append=True)



# -*- coding: utf-8 -*-
# @Time    : 2023/2/9 23:26
# @Author  : Euclid-Jie
# @File    : MongoClient.py
import pymongo


def MongoClient(DBName, collectionName):
    # 连接数据库
    myclient = pymongo.MongoClient("mongodb://localhost:27017/")
    mydb = myclient[DBName]  # 数据库名称
    mycol = mydb[collectionName]  # 集合（表）
    return mycol



# -*- coding: utf-8 -*-
# @Time    : 2023/2/9 21:36
# @Author  : Euclid-Jie
# @File    : Get_single_weibo_data.py


from Euclidweibo import *


def Get_single_weibo_data(mblogid):
    """
    get single weibo's data by weibo_url, just like https://weibo.com/1310272120/MrOtA75Fd
    which can get by using Get_item_url_list.py

    in fact get https://weibo.com/ajax/statuses/show?id=MrOtA75Fd
    con help to simplify works

    Attention: If need full content text, another get is needed https://weibo.com/ajax/statuses/longtext?id=MrOtA75Fd

    data contains:
        text: weibo content with some html
        text_raw: weibo content
        region_name: province

        attitudes_count = data['attitudes_count']
        comments_count = data['comments_count']
        reposts_count = data['reposts_count']

        mid: a id correspond to this weibo
        created_at: the time created this weibo
        source: the specific device information, just like "Phone 14 Pro"
        screen_name: the user who created this weibo
        ......
    """
    URL = 'https://weibo.com/ajax/statuses/show?id={}'.format(mblogid)
    current_dir = os.path.abspath(os.path.dirname(__file__))
    parent_dir = os.path.abspath(os.path.join(current_dir, os.pardir))
    header = Set_header(os.path.join(parent_dir, 'cookie.txt'))
    response = requests.get(URL, headers=header, timeout=60)  # 使用request获取网页
    html = response.content.decode('utf-8', 'ignore')  # 将网页源码转换格式为html
    data_json = json.loads(html, strict=False)
    if '展开' in data_json['text']:
        URL = 'https://weibo.com/ajax/statuses/longtext?id={}'.format(mblogid)
        response = requests.get(URL, headers=header, timeout=60)  # 使用request获取网页
        html = response.content.decode('utf-8', 'ignore')  # 将网页源码转换格式为html
        longTextContent = json.loads(html)['data']['longTextContent']
        data_json['longTextContent'] = longTextContent.encode('gbk', 'ignore').decode('gbk')
    else:
        data_json['longTextContent'] = ''
    return data_json


if __name__ == '__main__':
    data = Get_single_weibo_data(mblogid='MrOtA75Fd')
    part_data = {
        'time': data['created_at'],
        'mid': data['mid'],
        'nick_name': data['user']['screen_name'],
        'attitudes_count': data['attitudes_count'],
        'comments_count': data['comments_count'],
        'reposts_count': data['reposts_count'],
        'text': data['text'],
        'text_raw': data['text_raw'],
        'longTextContent': data['longTextContent']
    }
    print(part_data)



# -*- coding: utf-8 -*-
# @Time    : 2023/2/9 23:18
# @Author  : Euclid-Jie
# @File    : Get_longTextContent.py
import json
import requests
from Euclidweibo import *


def Get_longTextContent(data_json):
    if '展开' in data_json['text']:
        URL = 'https://weibo.com/ajax/statuses/longtext?id={}'.format(data_json['mblogid'])
        response = requests.get(URL, headers=Set_header('../cookie.txt'), timeout=60)  # 使用request获取网页
        html = response.content.decode('utf-8', 'ignore')  # 将网页源码转换格式为html
        longTextContent = json.loads(html)['data']['longTextContent']
        data_json['longTextContent'] = longTextContent.encode('gbk', 'ignore').decode('gbk')
    else:
        data_json['longTextContent'] = ''
    return data_json



# -*- coding: utf-8 -*-
# @Time    : 2023/2/9 20:01
# @Author  : Euclid-Jie
# @File    : Get_user_info.py
from Euclidweibo import *


def Get_user_info(uid):
    """
    base weibo user's uid, get his gender

    future can get user's more detail info

    base url : https://weibo.com/ajax/profile/info?uid=1202150843
    """

    URL = 'https://weibo.com/ajax/profile/info?uid={}'.format(uid)
    current_dir = os.path.abspath(os.path.dirname(__file__))
    parent_dir = os.path.abspath(os.path.join(current_dir, os.pardir))
    header = Set_header(os.path.join(parent_dir, 'cookie.txt'))
    response = requests.get(URL, headers=header, timeout=60)  # 使用request获取网页
    html = response.content.decode('utf-8', 'ignore')  # 将网页源码转换格式为html
    data_json = json.loads(html)['data']

    return data_json


if __name__ == '__main__':
    data = Get_user_info('7416119836', Set_header('../cookie.txt'))
    # followers_count
    print(data['user']['followers_count'])
    # gender
    print(data['user']['gender'])



# -*- coding: utf-8 -*-
# @Time    : 2023/2/9 20:42
# @Author  : Euclid-Jie
# @File    : Get_item_url_list.py
import re
import requests_html
from Euclidweibo import *


def Get_item_url_list(URL):
    """
    get all single weibo item 's url, just like https://weibo.com/1310272120/MrOtA75Fd
    1310272120 is uid
    MrOtA75Fd is mblogid
    use this mblogid, we get https://weibo.com/ajax/statuses/show?id=MrOtA75Fd
    con get all info about the single weibo
    """
    session = requests_html.HTMLSession()
    current_dir = os.path.abspath(os.path.dirname(__file__))
    parent_dir = os.path.abspath(os.path.join(current_dir, os.pardir))
    header = Set_header(os.path.join(parent_dir, 'cookie.txt'))
    response = session.get(URL, headers=header)
    response.encoding = 'utf-8'
    all_url_list = list(response.html.links)
    url_list = []
    pat = re.compile("\d{10}/[A-Za-z0-9]{9}")
    for url in all_url_list:
        tag = pat.findall(url)
        if tag:
            url_list.append(tag[0])

    return url_list


if __name__ == '__main__':
    url_list = Get_item_url_list('https://s.weibo.com/weibo?q=杭州公园')
    print(url_list)



# -*- coding: utf-8 -*-
# @Time    : 2023/3/21 21:35
# @Author  : Euclid-Jie
# @File    : Utils.py
import json

import requests


def Get_json_data(URL, header):
    response = requests.get(URL, headers=header, timeout=60)  # 使用request获取网页
    html = response.content.decode('utf-8', 'ignore')  # 将网页源码转换格式为html
    data_json = json.loads(html)
    return data_json



# -*- coding: utf-8 -*-
# @Time    : 2023/2/9 22:21
# @Author  : Euclid-Jie
# @File    : Get_single_weibo_details.py
import json
import requests
from tqdm import tqdm

import Euclidweibo
from Euclidweibo import *


class Get_single_weibo_details:
    """
    Usually a single weibo have three import information:
        1: reposts
        https://weibo.com/ajax/statuses/repostTimeline?id=4866288901690071&page=1&moduleID=feed&count=10
        2: comments
        https://weibo.com/ajax/statuses/buildComments?is_reload=1&id=4866288901690071&is_show_bulletin=2&is_mix=0&count=10&uid=1310272120
        3: attitudes
        https://weibo.com/ajax/statuses/likeShow?id=4866288901690071&attitude_type=0&attitude_enable=1&page=1&count=10

    Choose one or the other by setting para "method"
        method = ['reposts', 'comments', 'attitudes']
    """

    def __init__(self, method, mblogid, header):
        """
        :param method: ['reposts', 'comments', 'attitudes'],
        :param mblogid: just like "MrOtA75Fd",
        :param header: Set_header()
        :return: None
        """
        self.col = None
        self.page = None
        self.data_json = None
        self.item = None
        self.pages = None
        self.mid = None
        self.total_pages = None
        self.singe_weibo_data = None
        self.mblogid = mblogid
        self.method = method
        self.header = header
        # self.singe_weibo_data = Euclidweibo.Get_single_weibo_data(self.mblogid, self.header)
        # self.Get_single_weibo_infos()
        self.col = Euclidweibo.MongoClient('Weibo', mblogid + '_' + method)
        self.col = Euclidweibo.MongoClient('Weibo', '央视视频-主播说联播-media')

    def Get_single_weibo_infos(self):
        self.total_pages = {
            'reposts_count': self.singe_weibo_data['reposts_count'],
            'comments_count': self.singe_weibo_data['comments_count'],
            'attitudes_count': self.singe_weibo_data['attitudes_count']
        }

        self.mid = self.singe_weibo_data['mid']

    def Get_json_data(self, URL):
        response = requests.get(URL, headers=self.header, timeout=60)  # 使用request获取网页
        html = response.content.decode('utf-8', 'ignore')  # 将网页源码转换格式为html
        data_json = json.loads(html)
        return data_json

    def get_data_json(self):
        data_json = {
            'mblogid': self.item['mblogid'],
            'mid': self.item['id'],
            'time': self.item['created_at'],
            'uid': self.item['user']['id'],
            'nick_name': self.item['user']['screen_name'],
            'attitudes_count': self.item['attitudes_count'],
            'comments_count': self.item['comments_count'],
            'reposts_count': self.item['reposts_count'],
            'text': self.item['text'],
            'text_raw': self.item['text_raw'],
            'mp4_720p_mp4': self.item['page_info']['media_info']['mp4_720p_mp4'],
            'mp4_hd_url': self.item['page_info']['media_info']['mp4_hd_url'],
            'mp4_sd_url': self.item['page_info']['media_info']['mp4_sd_url'],
            'media_info': self.item['page_info']['media_info']
        }
        self.data_json = Euclidweibo.Get_longTextContent(data_json)

    def Get_reposts_info(self):
        # total_pages = int(self.total_pages['comments_count'] / 15)
        total_pages = 100
        with tqdm(range(18, total_pages + 1)) as t:
            for self.page in t:
                t.set_description("pages:{}".format(self.page))  # bar's left info
                # URL = 'https://weibo.com/ajax/statuses/repostTimeline?id={}&page={}&moduleID=feed&count=10'. \
                #     format(self.mid, self.pages)
                URL = 'https://weibo.com/ajax/profile/searchblog?uid=2656274875&page={}&feature=0&q=主播说联播'.format(self.page)
                data_json = self.Get_json_data(URL)
                reposts_list = data_json['data']['list']
                for item in reposts_list:
                    self.item = item
                    self.get_data_json()
                    # write to mongoDB
                    self.col.insert_one(self.data_json)
                t.set_postfix({"状态": "已成功写入{}条".format(len(reposts_list))})  # bar's right info

        return data_json

    def Get_comments_info(self):
        pass

    def Get_attitudes_info(self):
        pass

    def main_get(self):
        if self.method == 'reposts':
            self.Get_reposts_info()
        elif self.method == 'comments':
            self.Get_comments_info()
        elif self.method == 'attitudes':
            self.Get_attitudes_info()
        else:
            raise AttributeError('please set correct para method: reposts, comments or attitudes')


if __name__ == '__main__':
    Get_single_weibo_details('reposts', mblogid='MrOtA75Fd', header=Set_header('../cookie.txt')).main_get()



# -*- coding: utf-8 -*-
# @Time    : 2023/2/9 20:06
# @Author  : Euclid-Jie
# @File    : Set_header.py
import os


def Set_header(cookie_path):
    """
    设置header内容，可根据自己的更改cookie
    :param cookie_path: cookie path
    :return:
    """
    with open(cookie_path, 'rt', encoding='utf-8') as f:
        cookie = f.read().strip()
    header = {
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) '
                      'Chrome/104.0.0.0 Safari/537.36',
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,'
                  '*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        'accept-encoding': 'gzip, deflate, br',
        'Cookie': cookie
    }
    return header




# -*- coding: utf-8 -*-
# @Time    : 2023/3/11 11:16
# @Author  : Euclid-Jie
# @File    : EuclidDataTools_test.py
import pandas as pd

from EuclidDataTools import *

if __name__ == '__main__':
    data = {
        'id': 123,
        'sex': 'male',
        'age': '12',
        'job': 'student'
    }
    # 1、CsvClient.insert_one
    myCol = CsvClient(subFolder='demoOut', FileName='demo1')
    myCol.insert_one(data)

    # 2、EuclidCsvTools.saveCsvFile
    df = pd.DataFrame([data])
    EuclidCsvTools(subFolder='demoOut', FileName='demo2.csv').saveCsvFile(df, append=False)



# -*- coding: utf-8 -*-
# @Time    : 2023/2/10 10:52
# @Author  : Euclid-Jie
# @File    : __init__.py.py
import os
import json
import requests
from tqdm import tqdm

from .Get_single_weibo_details import Get_single_weibo_details
from .Set_header import Set_header
from .Get_single_weibo_data import Get_single_weibo_data
from .Get_longTextContent import Get_longTextContent
from .MongoClient import MongoClient
from .Get_item_url_list import Get_item_url_list
from .Get_user_info import Get_user_info
from .Get_user_all_weibo import *
from .Utils import *
from .EuclidDataTools import *



# -*- coding: utf-8 -*-
# @Time    : 2023/3/21 21:32
# @Author  : Euclid-Jie
# @File    : Get_user_all_weibo.py
import json
import requests
from tqdm import tqdm
import os
from Euclidweibo import *
from Euclidweibo.Utils import Get_json_data


def Get_user_all_weibo(uid, totalPages, begin=1, query=None, colName=None, csv=False):
    """
    the code is to get one's all or part blogs
    :param uid: the user's id
    :param totalPages: the total pages u want get, almost 20 blogs of each page
    :param begin: the beginning page u want to start
    :colName: the collection name u want to store
    """
    if not colName:
        colName = uid
    if not csv:
        _COL = MongoClient('Weibo', 'uid_{}'.format(colName))
    else:
        _COL = CsvClient('Weibo', 'uid_{}'.format(colName))
    current_dir = os.path.abspath(os.path.dirname(__file__))
    parent_dir = os.path.abspath(os.path.join(current_dir, os.pardir))
    _HEADER = Set_header(os.path.join(parent_dir, 'cookie.txt'))
    with tqdm(range(begin, totalPages)) as t:
        for pages in t:
            if query:
                URL = 'https://weibo.com/ajax/profile/searchblog?uid={}&page={}&feature=0&q={}'.format(uid, pages, query)
            else:
                URL = 'https://weibo.com/ajax/statuses/mymblog?uid={}&page={}'.format(uid, pages)
            data_json = Get_json_data(URL, _HEADER)
            for single_weibo in data_json['data']['list']:
                if '展开' in single_weibo['text']:
                    longText_URL = 'https://weibo.com/ajax/statuses/longtext?id={}'.format(single_weibo['mblogid'])
                    response = requests.get(longText_URL, headers=_HEADER, timeout=60)  # 使用request获取网页
                    html = response.content.decode('utf-8', 'ignore')  # 将网页源码转换格式为html
                    longTextContent = json.loads(html)['data']['longTextContent']
                    single_weibo['longTextContent'] = longTextContent
                else:
                    single_weibo['longTextContent'] = ''

                part_data = {
                    'time': single_weibo['created_at'],
                    'mid': single_weibo['mid'],
                    'nick_name': single_weibo['user']['screen_name'],
                    'attitudes_count': single_weibo['attitudes_count'],
                    'comments_count': single_weibo['comments_count'],
                    'reposts_count': single_weibo['reposts_count'],
                    'text': single_weibo['text'],
                    'text_raw': single_weibo['text_raw'],
                    'longTextContent': single_weibo['longTextContent'],
                    # 'online_users_number': single_weibo['page_info']['media_info']['online_users_number']
                }
                _COL.insert_one(part_data)
            t.set_postfix({"状态": "已写入{}条".format(len(data_json['data']['list']))})


if __name__ == '__main__':
    Get_user_all_weibo(2656274875, 100, query='主播说联播', colName='主播说联播', csv=True)
